@@ -9,24 +9,25 @@
 #include <iostream>
 #include <random>
 
-#include "config.h"
 #include "account.h"
+#include "accountmanager.h"
 #include "accountsetupcommandlinemanager.h"
 #include "accountstate.h"
-#include "editlocallymanager.h"
+#include "clientproxy.h"
+#include "config.h"
+#include "configfile.h"
 #include "connectionvalidator.h"
+#include "creds/abstractcredentials.h"
+#include "editlocallymanager.h"
 #include "folder.h"
 #include "folderman.h"
 #include "logger.h"
-#include "configfile.h"
+#include "pushnotifications.h"
+#include "shellextensionsserver.h"
 #include "socketapi/socketapi.h"
 #include "sslerrordialog.h"
 #include "theme.h"
-#include "clientproxy.h"
-#include "accountmanager.h"
-#include "creds/abstractcredentials.h"
-#include "pushnotifications.h"
-#include "shellextensionsserver.h"
+#include "updatechannel.h"
 
 #if defined(BUILD_UPDATER)
 #include "updater/ocupdater.h"
@@ -103,7 +104,7 @@ namespace {
         }
 #if defined(Q_OS_WIN)
         return QApplication::applicationDirPath() + QLatin1String("/i18n/");
-#elif defined(Q_OS_MAC)
+#elif defined(Q_OS_MACOS)
         return QApplication::applicationDirPath() + QLatin1String("/../Resources/Translations"); // path defaults to app dir.
 #elif defined(Q_OS_UNIX)
         if (qEnvironmentVariableIsSet("APPIMAGE")) {
@@ -131,10 +132,9 @@ bool Application::configVersionMigration()
     QStringList deleteKeys, ignoreKeys;
     AccountManager::backwardMigrationSettingsKeys(&deleteKeys, &ignoreKeys);
     FolderMan::backwardMigrationSettingsKeys(&deleteKeys, &ignoreKeys);
-    configFile.setClientPreviousVersionString(configFile.clientVersionString());
     
     qCDebug(lcApplication) << "Migration is in progress:"  << configFile.isMigrationInProgress();
-    const auto versionChanged = configFile.isUpgrade() || configFile.isDowngrade();
+    const auto versionChanged = configFile.hasVersionChanged();
     if (versionChanged) {
         qCInfo(lcApplication) << "Version changed. Removing updater settings from config.";
         configFile.cleanUpdaterConfiguration();
@@ -204,6 +204,7 @@ bool Application::configVersionMigration()
         }
     }
 
+    configFile.setClientPreviousVersionString(configFile.clientVersionString());
     configFile.setClientVersionString(MIRALL_VERSION_STRING);
     return true;
 }
@@ -294,9 +295,14 @@ Application::Application(int &argc, char **argv)
         setupConfigFile();
     }
 
+    // In the config, set the enterprise update channel to invalid, so it can be bumped up
+    // when recieving server capabilities.
+    ConfigFile().setDesktopEnterpriseChannel(UpdateChannel::Invalid.toString());
+
     if (_theme->doNotUseProxy()) {
         ConfigFile().setProxyType(QNetworkProxy::NoProxy);
-        for (const auto &accountState : AccountManager::instance()->accounts()) {
+        const auto &allAccounts = AccountManager::instance()->accounts();
+        for (const auto &accountState : allAccounts) {
             if (accountState && accountState->account()) {
                 accountState->account()->setProxyType(QNetworkProxy::NoProxy);
             }
@@ -376,7 +382,6 @@ Application::Application(int &argc, char **argv)
     }
 
     _theme->setSystrayUseMonoIcons(ConfigFile().monoIcons());
-    connect(_theme, &Theme::systrayUseMonoIconsChanged, _gui, &ownCloudGui::slotComputeOverallSyncStatus);
     connect(this, &Application::systemPaletteChanged,
             _theme, &Theme::systemPaletteHasChanged);
 
@@ -394,6 +399,7 @@ Application::Application(int &argc, char **argv)
     // Setting up the gui class will allow tray notifications for the
     // setup that follows, like folder setup
     _gui = new ownCloudGui(this);
+    connect(_theme, &Theme::systrayUseMonoIconsChanged, _gui, &ownCloudGui::slotComputeOverallSyncStatus);
     if (_showLogWindow) {
         _gui->slotToggleLogBrowser(); // _showLogWindow is set in parseOptions.
     }
@@ -404,7 +410,8 @@ Application::Application(int &argc, char **argv)
 
     if (_theme->doNotUseProxy()) {
         ConfigFile().setProxyType(QNetworkProxy::NoProxy);
-        for (const auto &accountState : AccountManager::instance()->accounts()) {
+        const auto &allAccounts = AccountManager::instance()->accounts();
+        for (const auto &accountState : allAccounts) {
             if (accountState && accountState->account()) {
                 accountState->account()->setProxyType(QNetworkProxy::NoProxy);
             }
@@ -488,12 +495,11 @@ void Application::setupAccountsAndFolders()
     ConfigFile configFile;
     configFile.setMigrationPhase(ConfigFile::MigrationPhase::SetupUsers);
     const auto accountsRestoreResult = restoreLegacyAccount();
-    const auto accounts = AccountManager::instance()->accounts();
-    if (accountsRestoreResult != AccountManager::AccountsRestoreSuccessFromLegacyVersion
-        && accounts.empty()) {
+    if (accountsRestoreResult == AccountManager::AccountsNotFound || accountsRestoreResult == AccountManager::AccountsRestoreFailure) {
         qCWarning(lcApplication) << "Migration result: " << accountsRestoreResult;
         qCDebug(lcApplication) << "is migration disabled?" << DISABLE_ACCOUNT_MIGRATION;
         qCWarning(lcApplication) << "No accounts were migrated, prompting user to set up accounts and folders from scratch.";
+        configFile.setMigrationPhase(ConfigFile::MigrationPhase::Done);
 
         return;
     }
@@ -510,10 +516,12 @@ void Application::setupAccountsAndFolders()
         return list.join("\n");
     };
 
+    const auto accounts = AccountManager::instance()->accounts();
     const auto accountsListSize = accounts.size();
     if (accountsRestoreResult == AccountManager::AccountsRestoreSuccessFromLegacyVersion
         && Theme::instance()->displayLegacyImportDialog()
-        && !AccountManager::instance()->forceLegacyImport()) {
+        && !AccountManager::instance()->forceLegacyImport()
+        && accountsListSize > 0) {
         const auto accountsRestoreMessage = accountsListSize > 1
             ? tr("%1 accounts", "number of accounts imported").arg(QString::number(accountsListSize))
             : tr("1 account");
