@@ -1,15 +1,6 @@
 /*
- * Copyright (C) by Claudio Cambra <claudio.cambra@nextcloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2022 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "editlocallyjob.h"
@@ -467,11 +458,11 @@ void EditLocallyJob::processLocalItem()
     if (rec.isDirectory() || !_accountState->account()->capabilities().filesLockAvailable()) {
         openFile();
     } else {
-        lockFile();
+        lockFile(rec._etag);
     }
 }
 
-void EditLocallyJob::lockFile()
+void EditLocallyJob::lockFile(const QString &etag)
 {
     Q_ASSERT(_accountState);
     Q_ASSERT(_accountState->account());
@@ -506,6 +497,7 @@ void EditLocallyJob::lockFile()
     _folderForFile->accountState()->account()->setLockFileState(_relPath,
                                                                 _folderForFile->remotePathTrailingSlash(),
                                                                 _folderForFile->path(),
+                                                                etag,
                                                                 _folderForFile->journalDb(),
                                                                 SyncFileItem::LockStatus::LockedItem,
                                                                 SyncFileItem::LockOwnerType::TokenLock);
@@ -513,7 +505,7 @@ void EditLocallyJob::lockFile()
 
 void EditLocallyJob::disconnectFolderSignals()
 {
-    for (const auto &connection : qAsConst(_folderConnections)) {
+    for (const auto &connection : std::as_const(_folderConnections)) {
         disconnect(connection);
     }
 }
