@@ -7,13 +7,15 @@
 
 #import <FileProvider/FileProvider.h>
 
+#include <QDir>
 #include <QLatin1StringView>
 #include <QLoggingCategory>
-#include <QRegularExpression>
+#include <QUuid>
 
 #include "config.h"
 #include "fileproviderdomainmanager.h"
 #include "fileprovidersettingscontroller.h"
+#include "fileproviderutils.h"
 
 #include "gui/accountmanager.h"
 #include "libsync/account.h"
@@ -28,59 +30,44 @@ Q_LOGGING_CATEGORY(lcMacFileProviderDomainManager, "nextcloud.gui.macfileprovide
 // are consistent throughout these classes
 namespace {
 
-static constexpr auto bundleExtensions = std::array{
-    QLatin1StringView(".app"),
-    QLatin1StringView(".framework"),
-    QLatin1StringView(".kext"),
-    QLatin1StringView(".plugin"),
-    QLatin1StringView(".docset"),
-    QLatin1StringView(".xpc"),
-    QLatin1StringView(".qlgenerator"),
-    QLatin1StringView(".component"),
-    QLatin1StringView(".saver"),
-    QLatin1StringView(".mdimporter")
-};
-static const QRegularExpression illegalChars("[:/]");
-
-inline bool hasBundleExtension(const QString &domainId)
-{
-    return std::any_of(bundleExtensions.begin(), bundleExtensions.end(), [&domainId](const auto &ext) {
-        return domainId.endsWith(ext);
-    });
-}
-
-inline bool illegalDomainIdentifier(const QString &domainId)
-{
-    return !domainId.isEmpty() && !illegalChars.match(domainId).hasMatch() && hasBundleExtension(domainId);
-}
-
-QString domainIdentifierForAccount(const OCC::Account * const account)
+QString uuidDomainIdentifierForAccount(const OCC::Account * const account)
 {
     Q_ASSERT(account);
-    auto domainId = account->userIdAtHostWithPort();
-    Q_ASSERT(!domainId.isEmpty());
+    const auto accountId = account->userIdAtHostWithPort();
 
-    domainId.replace(illegalChars, "-");
-
-    // Some url domains like .app cause issues on macOS as these are also bundle extensions.
-    // Under the hood, fileproviderd will create a folder for the user to access the files named
-    // after the domain identifier. If the url domain is the same as a bundle extension, Finder
-    // will interpret this folder as a bundle and will not allow the user to access the files.
-    // Here we wrap the dot in the url domain extension to prevent this from happening.
-    for (const auto &ext : bundleExtensions) {
-        if (domainId.endsWith(ext)) {
-            domainId = domainId.left(domainId.length() - ext.length());
-            domainId += "(.)" + ext.right(ext.length() - 1);
-            break;
-        }
+    if (accountId.isEmpty()) {
+        qCWarning(OCC::lcMacFileProviderDomainManager) << "Cannot generate UUID for account with empty userIdAtHostWithPort";
+        return {};
     }
 
-    return domainId;
+    // Try to get existing UUID mapping first
+    OCC::ConfigFile cfg;
+    const QString existingUuid = cfg.fileProviderDomainUuidFromAccountId(accountId);
+
+    if (!existingUuid.isEmpty()) {
+        qCDebug(OCC::lcMacFileProviderDomainManager) << "Using existing UUID for account:"
+                                                     << accountId
+                                                     << "UUID:"
+                                                     << existingUuid;
+
+        return existingUuid;
+    }
+
+    // Generate new UUID for this account
+    const QString newUuid = QUuid::createUuid().toString(QUuid::WithoutBraces);
+
+    qCInfo(OCC::lcMacFileProviderDomainManager) << "Generated new UUID for account:"
+                                                << accountId
+                                                << "UUID:"
+                                                << newUuid;
+
+    cfg.setFileProviderDomainUuidForAccountId(accountId, newUuid);
+    return newUuid;
 }
 
-inline QString domainIdentifierForAccount(const OCC::AccountPtr account)
+inline QString uuidDomainIdentifierForAccount(const OCC::AccountPtr account)
 {
-    return domainIdentifierForAccount(account.get());
+    return uuidDomainIdentifierForAccount(account.get());
 }
 
 inline QString domainDisplayNameForAccount(const OCC::Account * const account)
@@ -96,26 +83,90 @@ inline QString domainDisplayNameForAccount(const OCC::AccountPtr account)
 
 inline QString accountIdFromDomainId(const QString &domainId)
 {
+    if (domainId.isEmpty()) {
+        return {};
+    }
+
+    // Check if this is a UUID-based domain identifier
+    if (QUuid::fromString(domainId).isNull() == false) {
+        // This is a UUID, look up the account ID from the mapping
+        qCDebug(OCC::lcMacFileProviderDomainManager) << "Resolving UUID-based domain ID:"
+                                                     << domainId;
+
+        OCC::ConfigFile cfg;
+        const QString accountId = cfg.accountIdFromFileProviderDomainUuid(domainId);
+
+        if (!accountId.isEmpty()) {
+            qCDebug(OCC::lcMacFileProviderDomainManager) << "UUID maps to account:"
+                                                         << accountId;
+
+            return accountId;
+        }
+
+        qCWarning(OCC::lcMacFileProviderDomainManager) << "Could not find account id for UUID-based domain id:"
+                                                       << domainId;
+
+        return {};
+    }
+
+    // This is a legacy account-based domain identifier
+    qCDebug(OCC::lcMacFileProviderDomainManager) << "Using legacy account-based domain ID:"
+                                                 << domainId;
+
     return domainId;
 }
 
 QString accountIdFromDomainId(NSString * const domainId)
 {
+    if (!domainId) {
+        return {};
+    }
+
     auto qDomainId = QString::fromNSString(domainId);
+
+    if (qDomainId.isEmpty()) {
+        return {};
+    }
+
+    // Check if this is a UUID-based domain identifier
+    if (QUuid::fromString(qDomainId).isNull() == false) {
+        // This is a UUID, look up the account ID from the mapping
+        qCDebug(OCC::lcMacFileProviderDomainManager) << "Resolving UUID-based domain ID from NSString:" << qDomainId;
+        OCC::ConfigFile cfg;
+        const QString accountId = cfg.accountIdFromFileProviderDomainUuid(qDomainId);
+
+        if (!accountId.isEmpty()) {
+            qCDebug(OCC::lcMacFileProviderDomainManager) << "UUID maps to account:" << accountId;
+            return accountId;
+        }
+
+        qCWarning(OCC::lcMacFileProviderDomainManager) << "Could not find account id for UUID-based domain id:" << qDomainId;
+
+        return {};
+    }
+
+    // This is a legacy account-based domain identifier - handle the old logic
+    qCDebug(OCC::lcMacFileProviderDomainManager) << "Processing legacy account-based domain ID from NSString:" << qDomainId;
+
     if (!qDomainId.contains('-')) {
         return qDomainId.replace("(.)", ".");
     }
+
     // Using slashes as the replacement for illegal chars was unwise and we now have to pay the
     // price of doing so...
     const auto accounts = OCC::AccountManager::instance()->accounts();
+
     for (const auto &accountState : accounts) {
         const auto account = accountState->account();
-        const auto convertedDomainId = domainIdentifierForAccount(account);
+        const auto convertedDomainId = OCC::Mac::FileProviderUtils::domainIdentifierForAccount(account);
+
         if (convertedDomainId == qDomainId) {
             return account->userIdAtHostWithPort();
         }
     }
+
     qCWarning(OCC::lcMacFileProviderDomainManager) << "Could not find account id for domain id:" << qDomainId;
+
     return {};
 }
 
@@ -154,7 +205,7 @@ public:
                 }
 
                 if (domains.count == 0) {
-                    qCInfo(lcMacFileProviderDomainManager) << "Found no existing file provider domains";
+                    qCInfo(lcMacFileProviderDomainManager) << "Found no existing file provider domains at all.";
                     dispatch_group_leave(dispatchGroup);
                     return;
                 }
@@ -167,29 +218,38 @@ public:
                             accountState->account() &&
                             domainDisplayNameForAccount(accountState->account()) == QString::fromNSString(domain.displayName)) {
 
-                        qCInfo(lcMacFileProviderDomainManager) << "Found existing file provider domain for account:"
+                        qCInfo(lcMacFileProviderDomainManager) << "Found existing file provider domain with identifier:"
+                                                               << domain.identifier
+                                                               << "and display name:"
+                                                               << domain.displayName
+                                                               << "for account:"
                                                                << accountState->account()->displayName();
                         [domain retain];
 
-                        if (illegalDomainIdentifier(QString::fromNSString(domain.identifier))) {
+                        if (OCC::Mac::FileProviderUtils::illegalDomainIdentifier(QString::fromNSString(domain.identifier))) {
                             qCWarning(lcMacFileProviderDomainManager) << "Found existing file provider domain with illegal domain identifier:"
                                                                       << domain.identifier
+                                                                      << "and display name:"
+                                                                      << domain.displayName
                                                                       << "removing and recreating";
+
                             [NSFileProviderManager removeDomain:domain completionHandler:^(NSError * const error) {
                                 if (error) {
                                     qCWarning(lcMacFileProviderDomainManager) << "Error removing file provider domain with illegal domain identifier: "
                                                                               << error.code
                                                                               << error.localizedDescription;
-                                    
                                 } else {
                                     qCInfo(lcMacFileProviderDomainManager) << "Successfully removed file provider domain with illegal domain identifier: "
                                                                            << domain.identifier;
                                 }
+
+                                removeFileProviderDomainData(domain.identifier);
                                 [domain release];
                             }];
+
                             return;
                         }
-                        
+
                         _registeredDomains.insert(accountId, domain);
 
                         NSFileProviderManager * const fpManager = [NSFileProviderManager managerForDomain:domain];
@@ -203,7 +263,9 @@ public:
                             }
 
                             qCInfo(lcMacFileProviderDomainManager) << "Successfully reconnected file provider domain: "
-                                                                    << domain.displayName;
+                                                                   << domain.identifier
+                                                                   << "and display name:"
+                                                                   << domain.displayName;
                         }];
 
                     } else {
@@ -218,6 +280,8 @@ public:
                                                                           << error.code
                                                                           << error.localizedDescription;
                             }
+
+                            removeFileProviderDomainData(domain.identifier);
                         }];
                     }
                 }
@@ -236,15 +300,27 @@ public:
             const auto account = accountState->account();
             Q_ASSERT(account);
 
-            const auto domainId = domainIdentifierForAccount(account);
-            if (_registeredDomains.contains(domainId)) {
-                return _registeredDomains[domainId];
+            const auto accountId = account->userIdAtHostWithPort();
+            if (_registeredDomains.contains(accountId)) {
+                return _registeredDomains[accountId];
             }
         }
 
         return nil;
     }
 
+    QString fileProviderDomainIdentifierForAccountId(const QString &accountId)
+    {
+        if (@available(macOS 11.0, *)) {
+            if (_registeredDomains.contains(accountId)) {
+                const auto fileProviderDomain = _registeredDomains[accountId];
+                return QString::fromNSString([fileProviderDomain identifier]);
+            }
+        }
+
+        return {};
+    }
+
     void addFileProviderDomain(const AccountState * const accountState)
     {
         if (@available(macOS 11.0, *)) {
@@ -253,14 +329,18 @@ public:
             Q_ASSERT(account);
 
             const auto domainDisplayName = domainDisplayNameForAccount(account);
-            const auto domainId = domainIdentifierForAccount(account);
+            const auto domainId = uuidDomainIdentifierForAccount(account);  // Use UUID for new domains
+            const auto accountId = account->userIdAtHostWithPort();
 
-            qCInfo(lcMacFileProviderDomainManager) << "Adding new file provider domain with id: "
-                                                   << domainId;
+            qCInfo(lcMacFileProviderDomainManager) << "Adding new file provider domain with UUID: "
+                                                   << domainId
+                                                   << "for account:"
+                                                   << accountId;
 
-            if (_registeredDomains.contains(domainId) && _registeredDomains.value(domainId) != nil) {
-                qCDebug(lcMacFileProviderDomainManager) << "File provider domain with id already exists: "
-                                                        << domainId;
+            // Check if we already have a domain for this account (by account ID, not domain ID)
+            if (_registeredDomains.contains(accountId) && _registeredDomains.value(accountId) != nil) {
+                qCDebug(lcMacFileProviderDomainManager) << "File provider domain already exists for account: "
+                                                        << accountId;
                 return;
             }
 
@@ -274,11 +354,43 @@ public:
                                                               << error.localizedDescription;
                 }
 
-                _registeredDomains.insert(domainId, fileProviderDomain);
+                _registeredDomains.insert(accountId, fileProviderDomain);  // Store by account ID for easier lookup
             }];
         }
     }
 
+    void removeFileProviderDomainData(NSString * const domainIdentifier)
+    {
+        const auto qDomainIdentifier = QString::fromNSString(domainIdentifier);
+
+        // Remove logs.
+
+        auto logDirectory = OCC::Mac::FileProviderUtils::fileProviderDomainLogDirectory(qDomainIdentifier);
+
+        if (logDirectory.exists()) {
+            qCInfo(lcMacFileProviderDomainManager) << "Removing log directory at" << logDirectory.path();
+            logDirectory.removeRecursively();
+        } else {
+            qCInfo(lcMacFileProviderDomainManager) << "Due to lack of existence, not removing log directory at" << logDirectory.path();
+        }
+
+        // Remove support data.
+
+        auto supportDirectory = OCC::Mac::FileProviderUtils::fileProviderDomainSupportDirectory(qDomainIdentifier);
+
+        if (supportDirectory.exists()) {
+            qCInfo(lcMacFileProviderDomainManager) << "Removing support directory at" << supportDirectory.path();
+            supportDirectory.removeRecursively();
+        } else {
+            qCInfo(lcMacFileProviderDomainManager) << "Due to lack of existence, not removing support directory at" << supportDirectory.path();
+        }
+
+        // Remove configuration leftovers.
+
+        OCC::ConfigFile cfg;
+        cfg.removeFileProviderDomainMappingByDomainIdentifier(qDomainIdentifier);
+    }
+
     void removeFileProviderDomain(const AccountState * const accountState)
     {
         if (@available(macOS 11.0, *)) {
@@ -286,17 +398,17 @@ public:
             const auto account = accountState->account();
             Q_ASSERT(account);
 
-            const auto domainId = domainIdentifierForAccount(account);
-            qCInfo(lcMacFileProviderDomainManager) << "Removing file provider domain with id: "
-                                                   << domainId;
+            const auto accountId = account->userIdAtHostWithPort();
+            qCInfo(lcMacFileProviderDomainManager) << "Removing file provider domain for account: "
+                                                   << accountId;
 
-            if (!_registeredDomains.contains(domainId)) {
-                qCWarning(lcMacFileProviderDomainManager) << "File provider domain not found for id: "
-                                                          << domainId;
+            if (!_registeredDomains.contains(accountId)) {
+                qCWarning(lcMacFileProviderDomainManager) << "File provider domain not found for account: "
+                                                          << accountId;
                 return;
             }
 
-            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[domainId];
+            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[accountId];
 
             [NSFileProviderManager removeDomain:fileProviderDomain completionHandler:^(NSError *error) {
                 if (error) {
@@ -305,10 +417,13 @@ public:
                                                               << error.localizedDescription;
                 }
 
-                NSFileProviderDomain * const domain = _registeredDomains.take(domainId);
+                removeFileProviderDomainData(fileProviderDomain.identifier);
+                NSFileProviderDomain * const domain = _registeredDomains.take(accountId);
                 [domain release];
 
-                _registeredDomains.remove(domainId);
+                // Clean up the UUID mapping when removing the domain
+                OCC::ConfigFile cfg;
+                cfg.removeFileProviderDomainUuidMapping(accountId);
             }];
         }
     }
@@ -327,11 +442,16 @@ public:
                 }
 
                 const auto registeredDomainPtrs = _registeredDomains.values();
+                const auto accountIds = _registeredDomains.keys();
+
                 for (NSFileProviderDomain * const domain : registeredDomainPtrs) {
+                    removeFileProviderDomainData(domain.identifier);
+
                     if (domain != nil) {
                         [domain release];
                     }
                 }
+
                 _registeredDomains.clear();
             }];
         }
@@ -362,7 +482,11 @@ public:
                             return;
                         }
 
-                        NSFileProviderDomain * const registeredDomainPtr = _registeredDomains.take(QString::fromNSString(domain.identifier));
+                        removeFileProviderDomainData(domain.identifier);
+
+                        const QString accountId = accountIdFromDomainId(domain.identifier);
+                        NSFileProviderDomain * const registeredDomainPtr = _registeredDomains.take(accountId);
+
                         if (registeredDomainPtr != nil) {
                             [domain release];
                         }
@@ -382,17 +506,17 @@ public:
             const auto account = accountState->account();
             Q_ASSERT(account);
 
-            const auto domainId = domainIdentifierForAccount(account);
-            qCInfo(lcMacFileProviderDomainManager) << "Disconnecting file provider domain with id: "
-                                                   << domainId;
+            const auto accountId = account->userIdAtHostWithPort();
+            qCInfo(lcMacFileProviderDomainManager) << "Disconnecting file provider domain for account: "
+                                                   << accountId;
 
-            if(!_registeredDomains.contains(domainId)) {
-                qCInfo(lcMacFileProviderDomainManager) << "File provider domain not found for id: "
-                                                       << domainId;
+            if(!_registeredDomains.contains(accountId)) {
+                qCInfo(lcMacFileProviderDomainManager) << "File provider domain not found for account: "
+                                                       << accountId;
                 return;
             }
 
-            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[domainId];
+            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[accountId];
             Q_ASSERT(fileProviderDomain != nil);
 
             NSFileProviderManager * const fpManager = [NSFileProviderManager managerForDomain:fileProviderDomain];
@@ -420,20 +544,21 @@ public:
             const auto account = accountState->account();
             Q_ASSERT(account);
 
-            const auto domainId = domainIdentifierForAccount(account);
-            qCInfo(lcMacFileProviderDomainManager) << "Reconnecting file provider domain with id: "
-                                                   << domainId;
+            const auto accountId = account->userIdAtHostWithPort();
+            qCInfo(lcMacFileProviderDomainManager) << "Reconnecting file provider domain for account: "
+                                                   << accountId;
 
-            if(!_registeredDomains.contains(domainId)) {
-                qCInfo(lcMacFileProviderDomainManager) << "File provider domain not found for id: "
-                                                       << domainId;
+            if(!_registeredDomains.contains(accountId)) {
+                qCInfo(lcMacFileProviderDomainManager) << "File provider domain not found for account: "
+                                                       << accountId;
                 return;
             }
 
-            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[domainId];
+            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[accountId];
             Q_ASSERT(fileProviderDomain != nil);
 
             NSFileProviderManager * const fpManager = [NSFileProviderManager managerForDomain:fileProviderDomain];
+
             [fpManager reconnectWithCompletionHandler:^(NSError * const error) {
                 if (error) {
                     qCWarning(lcMacFileProviderDomainManager) << "Error reconnecting file provider domain: "
@@ -455,18 +580,18 @@ public:
     {
         if (@available(macOS 11.0, *)) {
             Q_ASSERT(account);
-            const auto domainId = domainIdentifierForAccount(account);
+            const auto accountId = account->userIdAtHostWithPort();
 
-            qCInfo(lcMacFileProviderDomainManager) << "Signalling enumerator changed in file provider domain for account with id: "
-                                                   << domainId;
+            qCInfo(lcMacFileProviderDomainManager) << "Signalling enumerator changed in file provider domain for account: "
+                                                   << accountId;
 
-            if(!_registeredDomains.contains(domainId)) {
-                qCInfo(lcMacFileProviderDomainManager) << "File provider domain not found for id: "
-                                                       << domainId;
+            if(!_registeredDomains.contains(accountId)) {
+                qCInfo(lcMacFileProviderDomainManager) << "File provider domain not found for account: "
+                                                       << accountId;
                 return;
             }
 
-            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[domainId];
+            NSFileProviderDomain * const fileProviderDomain = _registeredDomains[accountId];
             Q_ASSERT(fileProviderDomain != nil);
 
             NSFileProviderManager * const fpManager = [NSFileProviderManager managerForDomain:fileProviderDomain];
@@ -479,12 +604,13 @@ public:
         }
     }
 
-    QStringList configuredDomainIds() const
+    QStringList getAccountIdsOfFoundFileProviderDomains() const
     {
         return _registeredDomains.keys();
     }
 
 private:
+    //! keys are accountId, i.e. userIdAtHostWithPort
     QHash<QString, NSFileProviderDomain*> _registeredDomains;
 };
 
@@ -511,6 +637,7 @@ void FileProviderDomainManager::start()
     // shutdowns.
     connect(AccountManager::instance(), &AccountManager::accountSyncConnectionRemoved,
             this, &FileProviderDomainManager::removeFileProviderDomainForAccount);
+
     connect(AccountManager::instance(), &AccountManager::accountRemoved,
             this, [this](const AccountState * const accountState) {
         const auto trReason = tr("%1 application has been closed. Reopen to reconnect.").arg(APPLICATION_NAME);
@@ -533,34 +660,73 @@ void FileProviderDomainManager::setupFileProviderDomains()
 
 void FileProviderDomainManager::updateFileProviderDomains()
 {
+    qCDebug(lcMacFileProviderDomainManager) << "Updating file provider domains.";
+
     if (!d) {
         return;
     }
 
-    const auto vfsEnabledAccounts = FileProviderSettingsController::instance()->vfsEnabledAccounts();
-    auto configuredDomains = d->configuredDomainIds();
+    const auto fileProviderEnabledAccountIds = FileProviderSettingsController::instance()->vfsEnabledAccounts();
+    auto accountIdsOfFoundFileProviderDomains = d->getAccountIdsOfFoundFileProviderDomains();
 
-    for (const auto &accountUserIdAtHost : vfsEnabledAccounts) {
-        // If the domain has already been set up for this account, then don't set it up again
-        if (configuredDomains.contains(accountUserIdAtHost)) {
-            configuredDomains.removeAll(accountUserIdAtHost);
+    for (const auto &fileProviderEnabledAccountId : fileProviderEnabledAccountIds) {
+        // If the domain has already been set up for this account, then don't set it up again.
+        if (accountIdsOfFoundFileProviderDomains.contains(fileProviderEnabledAccountId)) {
+            accountIdsOfFoundFileProviderDomains.removeAll(fileProviderEnabledAccountId);
             continue;
         }
 
-        if (const auto accountState = AccountManager::instance()->accountFromUserId(accountUserIdAtHost)) {
+        if (const auto accountState = AccountManager::instance()->accountFromUserId(fileProviderEnabledAccountId)) {
+            qCDebug(lcMacFileProviderDomainManager) << "Succeed in fetching account state by account id"
+                                                    << fileProviderEnabledAccountId
+                                                    << ", adding file provider domain for account.";
+
             addFileProviderDomainForAccount(accountState.data());
         } else {
-            qCWarning(lcMacFileProviderDomainManager) << "Could not find account for file provider domain:" << accountUserIdAtHost
-                                                      << "removing account from list of vfs-enabled accounts.";
-            FileProviderSettingsController::instance()->setVfsEnabledForAccount(accountUserIdAtHost, false);
+            qCWarning(lcMacFileProviderDomainManager) << "Could not fetch account state by account id"
+                                                      << fileProviderEnabledAccountId
+                                                      << ", removing account from list of VFS-enabled accounts.";
+
+            FileProviderSettingsController::instance()->setVfsEnabledForAccount(fileProviderEnabledAccountId, false);
         }
     }
 
-    for (const auto &remainingDomainUserId : configuredDomains) {
-        const auto accountState = AccountManager::instance()->accountFromUserId(remainingDomainUserId);
+    for (const auto &remainingAccountId : accountIdsOfFoundFileProviderDomains) {
+        qCDebug(lcMacFileProviderDomainManager) << "Orphaned file provider domain to remove found for account id"
+                                                << remainingAccountId;
+
+        const auto accountState = AccountManager::instance()->accountFromUserId(remainingAccountId);
         removeFileProviderDomainForAccount(accountState.data());
     }
 
+    // If there are no enabled accounts, check for and remove the FileProviderExt directory
+    // from the group container
+    if (fileProviderEnabledAccountIds.isEmpty()) {
+        const QString groupContainer = Mac::FileProviderUtils::groupContainerPath();
+
+        if (!groupContainer.isEmpty()) {
+            QDir groupDir(groupContainer);
+            const QString fileProviderExtDirName = QStringLiteral("FileProviderExt");
+
+            if (groupDir.exists(fileProviderExtDirName)) {
+                const QString fileProviderExtPath = groupDir.absoluteFilePath(fileProviderExtDirName);
+                QDir fileProviderExtDir(fileProviderExtPath);
+
+                qCInfo(lcMacFileProviderDomainManager) << "No file provider enabled accounts, removing FileProviderExt directory at:"
+                                                       << fileProviderExtPath;
+
+                if (fileProviderExtDir.removeRecursively()) {
+                    qCInfo(lcMacFileProviderDomainManager) << "Successfully removed FileProviderExt directory in groun container.";
+                } else {
+                    qCWarning(lcMacFileProviderDomainManager) << "Failed to remove FileProviderExt directory at:"
+                                                              << fileProviderExtPath;
+                }
+            }
+        } else {
+            qCWarning(lcMacFileProviderDomainManager) << "Could not determine group container path!";
+        }
+    }
+
     emit domainSetupComplete();
 }
 
@@ -599,9 +765,6 @@ void FileProviderDomainManager::removeFileProviderDomainForAccount(const Account
     }
 
     Q_ASSERT(accountState);
-    const auto account = accountState->account();
-    Q_ASSERT(account);
-
     d->removeFileProviderDomain(accountState);
 }
 
@@ -685,9 +848,13 @@ AccountStatePtr FileProviderDomainManager::accountStateFromFileProviderDomainIde
     return accountForReceivedDomainIdentifier;
 }
 
-QString FileProviderDomainManager::fileProviderDomainIdentifierFromAccountState(const AccountStatePtr &accountState)
+QString FileProviderDomainManager::fileProviderDomainIdentifierFromAccountId(const QString &accountId)
 {
-    return domainIdentifierForAccount(accountState->account());
+    if (!d || accountId.isEmpty()) {
+        return {};
+    }
+
+    return d->fileProviderDomainIdentifierForAccountId(accountId);
 }
 
 void* FileProviderDomainManager::domainForAccount(const AccountState * const accountState)
