@@ -24,12 +24,14 @@
 #include "tray/talkreply.h"
 #include "userstatusconnector.h"
 
+#include <QtCore>
 #include <QDesktopServices>
 #include <QIcon>
 #include <QMessageBox>
 #include <QSvgRenderer>
 #include <QPainter>
 #include <QPushButton>
+#include <QDateTime>
 
 // time span in milliseconds which has to be between two
 // refreshes of the notifications
@@ -83,6 +85,7 @@ User::User(AccountStatePtr &account, const bool &isCurrent, QObject *parent)
     connect(_account.data(), &AccountState::hasFetchedNavigationApps,
         this, &User::slotRebuildNavigationAppList);
     connect(_account->account().data(), &Account::accountChangedDisplayName, this, &User::nameChanged);
+    connect(_account->account().data(), &Account::rootFolderQuotaChanged, this, &User::slotQuotaChanged);
 
     connect(FolderMan::instance(), &FolderMan::folderListChanged, this, &User::hasLocalFolderChanged);
 
@@ -897,6 +900,16 @@ const QVariantList &User::groupFolders() const
     return _trayFolderInfos;
 }
 
+bool User::canLogout() const
+{
+    return !isPublicShareLink();
+}
+
+bool User::isPublicShareLink() const
+{
+    return _account->account()->isPublicShareLink();
+}
+
 void User::slotItemCompleted(const QString &folder, const SyncFileItemPtr &item)
 {
     auto folderInstance = FolderMan::instance()->folder(folder);
@@ -1001,17 +1014,27 @@ void User::logout() const
 
 QString User::name() const
 {
+    if (isPublicShareLink()) {
+        return tr("Public Share Link");
+    }
+
     return _account->account()->prettyName();
 }
 
 QString User::server(bool shortened) const
 {
-    QString serverUrl = _account->account()->url().toString();
-    if (shortened) {
-        serverUrl.replace(QLatin1String("https://"), QLatin1String(""));
-        serverUrl.replace(QLatin1String("http://"), QLatin1String(""));
+    auto serverUrl = _account->account()->url();
+
+    if (isPublicShareLink()) {
+        serverUrl.setUserName({});
     }
-    return serverUrl;
+    QString stringServerUrl = serverUrl.toString();
+    if (shortened) {
+        stringServerUrl.replace(QLatin1String("https://"), QLatin1String(""));
+        stringServerUrl.replace(QLatin1String("http://"), QLatin1String(""));
+
+    }
+    return stringServerUrl;
 }
 
 UserStatus::OnlineStatus User::status() const
@@ -1076,7 +1099,9 @@ QString User::featuredAppIcon() const
 
 QString User::featuredAppAccessibleName() const
 {
-    return isNcAssistantEnabled() ? tr("Open Nextcloud Assistant in browser") : tr("Open Nextcloud Talk in browser");
+    return isNcAssistantEnabled() ?
+        tr("Open %1 Assistant in browser", "The placeholder will be the application name. Please keep it").arg(APPLICATION_NAME) :
+        tr("Open %1 Talk in browser", "The placeholder will be the application name. Please keep it").arg(APPLICATION_NAME);
 }
 
 AccountApp *User::talkApp() const
@@ -1182,6 +1207,52 @@ void User::slotFetchGroupFolders()
     connect(groupFolderListJob, &SimpleNetworkJob::finishedSignal, this, &User::slotGroupFoldersFetched);
 }
 
+void User::slotQuotaChanged(const int64_t &usedBytes, const int64_t &availableBytes)
+{
+    if (availableBytes < 0) {
+        // values less than 0 -> quota is not set or determinable
+        // just reset the status
+        _lastQuotaPercent = 0;
+        _activityModel->removeActivityFromActivityList(_lastQuotaActivity);
+        return;
+    }
+
+    int64_t total = usedBytes + availableBytes;
+    if (total <= 0 || !ConfigFile().showQuotaWarningNotifications()) {
+        return;
+    }
+
+    const auto percent = (double)usedBytes / (double)total * 100.0;
+    const auto percentInt = qMin(qRound(percent), 100);
+    qCDebug(lcActivity) << tr("Quota is updated; %1 percent of the total space is used.").arg(QString::number(percentInt));
+
+    int thresholdPassed = 0;
+    if (_lastQuotaPercent < 80 && percentInt >= 80) {
+        thresholdPassed = 80;
+    }
+
+    if (_lastQuotaPercent < 90 && percentInt >= 90) {
+        thresholdPassed = 90;
+    }
+
+    if (_lastQuotaPercent < 95 && percentInt >= 95) {
+        thresholdPassed = 95;
+    }
+
+    if (thresholdPassed > 0) {
+        _activityModel->removeActivityFromActivityList(_lastQuotaActivity);
+
+        _lastQuotaActivity._type = Activity::OpenSettingsNotificationType;
+        _lastQuotaActivity._dateTime = QDateTime::fromString(QDateTime::currentDateTime().toString(), Qt::ISODate);
+        _lastQuotaActivity._subject = tr("Quota Warning - %1 percent or more storage in use").arg(QString::number(thresholdPassed));
+        _lastQuotaActivity._accName = account()->displayName();
+        _lastQuotaActivity._id = qHash(QDateTime::currentMSecsSinceEpoch());
+        showDesktopNotification(_lastQuotaActivity);
+        _activityModel->addNotificationToActivityList(_lastQuotaActivity);
+    }
+    _lastQuotaPercent = percentInt;
+}
+
 void User::slotGroupFoldersFetched(QNetworkReply *reply)
 {
     Q_ASSERT(reply);
@@ -1217,7 +1288,7 @@ void User::slotGroupFoldersFetched(QNetworkReply *reply)
     const auto obj = json.object().toVariantMap();
     const auto groupFolders = obj["ocs"].toMap()["data"].toMap();
 
-    for (const auto &groupFolder : groupFolders.values()) {
+    for (const auto &groupFolder : groupFolders) {
         const auto groupFolderInfo = groupFolder.toMap();
         const auto mountPoint = groupFolderInfo.value(QStringLiteral("mount_point"), {}).toString();
         parseNewGroupFolderPath(mountPoint);
@@ -1528,36 +1599,54 @@ int UserModel::rowCount(const QModelIndex &parent) const
 
 QVariant UserModel::data(const QModelIndex &index, int role) const
 {
-    if (index.row() < 0 || index.row() >= _users.count()) {
-        return QVariant();
+    auto result = QVariant{};
+    switch (static_cast<UserRoles>(role))
+    {
+    case NameRole:
+        result = _users[index.row()]->name();
+        break;
+    case ServerRole:
+        result = _users[index.row()]->server();
+        break;
+    case ServerHasUserStatusRole:
+        result = _users[index.row()]->serverHasUserStatus();
+        break;
+    case StatusRole:
+        result = QVariant::fromValue(_users[index.row()]->status());
+        break;
+    case StatusIconRole:
+        result = _users[index.row()]->statusIcon();
+        break;
+    case StatusEmojiRole:
+        result = _users[index.row()]->statusEmoji();
+        break;
+    case StatusMessageRole:
+        result = _users[index.row()]->statusMessage();
+        break;
+    case DesktopNotificationsAllowedRole:
+        result = _users[index.row()]->isDesktopNotificationsAllowed();
+        break;
+    case AvatarRole:
+        result = _users[index.row()]->avatarUrl();
+        break;
+    case IsCurrentUserRole:
+        result = _users[index.row()]->isCurrentUser();
+        break;
+    case IsConnectedRole:
+        result = _users[index.row()]->isConnected();
+        break;
+    case IdRole:
+        result = index.row();
+        break;
+    case CanLogoutRole:
+        result = _users[index.row()]->canLogout();
+        break;
+    case RemoveAccountTextRole:
+        result = _users[index.row()]->isPublicShareLink() ? tr("Leave share") : tr("Remove account");
+        break;
     }
 
-    if (role == NameRole) {
-        return _users[index.row()]->name();
-    } else if (role == ServerRole) {
-        return _users[index.row()]->server();
-    } else if (role == ServerHasUserStatusRole) {
-        return _users[index.row()]->serverHasUserStatus();
-    } else if (role == StatusRole) {
-        return QVariant::fromValue(_users[index.row()]->status());
-    } else if (role == StatusIconRole) {
-        return _users[index.row()]->statusIcon();
-    } else if (role == StatusEmojiRole) {
-        return _users[index.row()]->statusEmoji();
-    } else if (role == StatusMessageRole) {
-        return _users[index.row()]->statusMessage();
-    } else if (role == DesktopNotificationsAllowedRole) {
-        return _users[index.row()]->isDesktopNotificationsAllowed();
-    } else if (role == AvatarRole) {
-        return _users[index.row()]->avatarUrl();
-    } else if (role == IsCurrentUserRole) {
-        return _users[index.row()]->isCurrentUser();
-    } else if (role == IsConnectedRole) {
-        return _users[index.row()]->isConnected();
-    } else if (role == IdRole) {
-        return index.row();
-    }
-    return QVariant();
+    return result;
 }
 
 QHash<int, QByteArray> UserModel::roleNames() const
@@ -1575,6 +1664,8 @@ QHash<int, QByteArray> UserModel::roleNames() const
     roles[IsCurrentUserRole] = "isCurrentUser";
     roles[IsConnectedRole] = "isConnected";
     roles[IdRole] = "id";
+    roles[CanLogoutRole] = "canLogout";
+    roles[RemoveAccountTextRole] = "removeAccountText";
     return roles;
 }
 
@@ -1745,7 +1836,8 @@ void UserAppsModel::buildAppList()
 
     if (UserModel::instance()->appList().count() > 0) {
         const auto talkApp = UserModel::instance()->currentUser()->talkApp();
-        for (const auto &app : UserModel::instance()->appList()) {
+        const auto &allApps = UserModel::instance()->appList();
+        for (const auto &app : allApps) {
             // Filter out Talk because we have a dedicated button for it
             if (talkApp && app->id() == talkApp->id() && !UserModel::instance()->currentUser()->isNcAssistantEnabled()) {
                 continue;
