@@ -5,14 +5,20 @@
 
 #include "cfapiwrapper.h"
 
+#include "config.h"
+
 #include "common/utility.h"
 #include "common/filesystembase.h"
 #include "hydrationjob.h"
 #include "theme.h"
 #include "vfs_cfapi.h"
+#include "accessmanager.h"
 
 #include <QCoreApplication>
 #include <QEventLoop>
+#include <QPromise>
+#include <QFuture>
+#include <QFutureWatcher>
 #include <QDir>
 #include <QFileInfo>
 #include <QLocalSocket>
@@ -23,8 +29,7 @@
 #include <ntstatus.h>
 #include <cfapi.h>
 #include <comdef.h>
-
-#include "config.h"
+#include <memory>
 
 Q_LOGGING_CATEGORY(lcCfApiWrapper, "nextcloud.sync.vfs.cfapi.wrapper", QtInfoMsg)
 using namespace Qt::Literals::StringLiterals;
@@ -126,6 +131,78 @@ void cfApiSendTransferInfo(const CF_CONNECTION_KEY &connectionKey, const CF_TRAN
     }
 }
 
+void cfApiSendPlaceholdersTransferInfo(const CF_CONNECTION_KEY &connectionKey,
+                                       const CF_TRANSFER_KEY &transferKey,
+                                       NTSTATUS status,
+                                       const QList<OCC::PlaceholderCreateInfo> &newEntries,
+                                       qint64 currentPlaceholdersCount,
+                                       qint64 totalPlaceholdersCount,
+                                       const QString &serverPath)
+{
+    CF_OPERATION_INFO opInfo = { 0 };
+    CF_OPERATION_PARAMETERS opParams = { 0 };
+
+    const auto newPlaceholders = std::make_unique<CF_PLACEHOLDER_CREATE_INFO[]>(newEntries.size());
+
+    for (auto i = 0; i < newEntries.size(); ++i) {
+        const auto &entryInfo = newEntries[i];
+        auto &newPlaceholder = newPlaceholders[i];
+
+        qCInfo(lcCfApiWrapper()) << entryInfo.name
+                                 << "fileId:" << entryInfo.parsedProperties.fileId
+                                 << "fileSize:" << entryInfo.parsedProperties.size
+                                 << "fileMtime:" << entryInfo.parsedProperties.modtime
+                                 << "fileResourceType:" << (entryInfo.parsedProperties.isDirectory ? "folder" : "file");
+
+        newPlaceholder.RelativeFileName = entryInfo.stdWStringName.c_str();
+        const auto fileIdentity = entryInfo.parsedProperties.fileId;
+        newPlaceholder.FileIdentity = fileIdentity.data();
+        newPlaceholder.FileIdentityLength = (fileIdentity.length() + 1) * sizeof(wchar_t);
+        newPlaceholder.Flags = CF_PLACEHOLDER_CREATE_FLAG_MARK_IN_SYNC;
+        auto &fsMetadata = newPlaceholder.FsMetadata;
+
+        fsMetadata.FileSize.QuadPart = entryInfo.parsedProperties.size;
+        fsMetadata.BasicInfo.FileAttributes = FILE_ATTRIBUTE_NORMAL;
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(entryInfo.parsedProperties.modtime, &fsMetadata.BasicInfo.CreationTime);
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(entryInfo.parsedProperties.modtime, &fsMetadata.BasicInfo.LastWriteTime);
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(entryInfo.parsedProperties.modtime, &fsMetadata.BasicInfo.LastAccessTime);
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(entryInfo.parsedProperties.modtime, &fsMetadata.BasicInfo.ChangeTime);
+
+        if (entryInfo.parsedProperties.isDirectory) {
+            fsMetadata.BasicInfo.FileAttributes = FILE_ATTRIBUTE_DIRECTORY;
+            fsMetadata.FileSize.QuadPart = 0;
+        }
+    }
+
+    opInfo.StructSize = sizeof(opInfo);
+    opInfo.Type = CF_OPERATION_TYPE_TRANSFER_PLACEHOLDERS;
+    opInfo.ConnectionKey = connectionKey;
+    opInfo.TransferKey = transferKey;
+
+    opParams.ParamSize = CF_SIZE_OF_OP_PARAM(TransferPlaceholders);
+    opParams.TransferPlaceholders.Flags = CF_OPERATION_TRANSFER_PLACEHOLDERS_FLAG_DISABLE_ON_DEMAND_POPULATION;
+    opParams.TransferPlaceholders.CompletionStatus = status;
+
+    if (!newEntries.isEmpty()) {
+        opParams.TransferPlaceholders.PlaceholderTotalCount.QuadPart = totalPlaceholdersCount;
+        opParams.TransferPlaceholders.PlaceholderCount = currentPlaceholdersCount;
+        opParams.TransferPlaceholders.EntriesProcessed = currentPlaceholdersCount;
+        opParams.TransferPlaceholders.PlaceholderArray = newPlaceholders.get();
+    } else {
+        opParams.TransferPlaceholders.PlaceholderTotalCount.QuadPart = 0;
+        opParams.TransferPlaceholders.PlaceholderCount = 0;
+        opParams.TransferPlaceholders.EntriesProcessed = 0;
+        opParams.TransferPlaceholders.PlaceholderArray = nullptr;
+    }
+
+    const qint64 cfExecuteresult = CfExecute(&opInfo, &opParams);
+    if (cfExecuteresult != S_OK) {
+        qCCritical(lcCfApiWrapper) << "Couldn't send transfer info" << QString::number(transferKey.QuadPart, 16) << ":" << cfExecuteresult << QString::fromWCharArray(_com_error(cfExecuteresult).ErrorMessage());
+    }
+
+    qCInfo(lcCfApiWrapper()) << "number of processes entries:" << opParams.TransferPlaceholders.EntriesProcessed;
+}
+
 void CALLBACK cfApiFetchDataCallback(const CF_CALLBACK_INFO *callbackInfo, const CF_CALLBACK_PARAMETERS *callbackParameters)
 {
     qCDebug(lcCfApiWrapper) << "Fetch data callback called. File size:" << callbackInfo->FileSize.QuadPart;
@@ -313,12 +390,14 @@ enum class CfApiUpdateMetadataType {
 };
 
 OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> updatePlaceholderState(const QString &path,
-                                                                                  time_t modtime,
-                                                                                  qint64 size,
-                                                                                  const QByteArray &fileId,
+                                                                                  const OCC::SyncFileItem &item,
                                                                                   const QString &replacesPath,
                                                                                   CfApiUpdateMetadataType updateType)
 {
+    const time_t modtime = item._modtime;
+    const qint64 size = item._size;
+    const QByteArray &fileId = item._fileId;
+
     if (updateType == CfApiUpdateMetadataType::AllMetadata && modtime <= 0) {
         return {QString{"Could not update metadata due to invalid modification time for %1: %2"}.arg(path).arg(modtime)};
     }
@@ -338,16 +417,16 @@ OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> updatePlaceholderStat
     OCC::Utility::UnixTimeToLargeIntegerFiletime(modtime, &metadata.BasicInfo.LastAccessTime);
     OCC::Utility::UnixTimeToLargeIntegerFiletime(modtime, &metadata.BasicInfo.ChangeTime);
 
-
     qCInfo(lcCfApiWrapper) << "updatePlaceholderState" << path << modtime;
-    const qint64 result =
-        CfUpdatePlaceholder(OCC::CfApiWrapper::handleForPath(path).get(), updateType == CfApiUpdateMetadataType::AllMetadata ? &metadata : nullptr,
-                            fileId.data(), static_cast<DWORD>(fileId.size()), nullptr, 0, CF_UPDATE_FLAG_MARK_IN_SYNC, nullptr, nullptr);
+    const auto updateFlags = CF_UPDATE_FLAG_MARK_IN_SYNC; // item.isDirectory() ? CF_UPDATE_FLAG_MARK_IN_SYNC | CF_UPDATE_FLAG_ENABLE_ON_DEMAND_POPULATION : CF_UPDATE_FLAG_MARK_IN_SYNC;
+
+    const auto result = CfUpdatePlaceholder(OCC::CfApiWrapper::handleForPath(path).get(), updateType == CfApiUpdateMetadataType::AllMetadata ? &metadata : nullptr,
+                                              fileId.data(), static_cast<DWORD>(fileId.size()),
+                                              nullptr, 0, updateFlags, nullptr, nullptr);
 
     if (result != S_OK) {
         const auto errorMessage = createErrorMessageForPlaceholderUpdateAndCreate(path, "Couldn't update placeholder info");
         qCWarning(lcCfApiWrapper) << errorMessage << path << ":" << QString::fromWCharArray(_com_error(result).ErrorMessage()) << replacesPath;
-        return errorMessage;
     }
 
            // Pin state tends to be lost on updates, so restore it every time
@@ -428,6 +507,122 @@ void CALLBACK cfApiCancelFetchPlaceHolders(const CF_CALLBACK_INFO *callbackInfo,
     }
 }
 
+void CALLBACK cfApiFetchPlaceHolders(const CF_CALLBACK_INFO *callbackInfo, const CF_CALLBACK_PARAMETERS *callbackParameters)
+{
+    const auto path = QString(QString::fromWCharArray(callbackInfo->VolumeDosName) + QString::fromWCharArray(callbackInfo->NormalizedPath));
+
+    qDebug(lcCfApiWrapper) << "Fetch placeholders callback called. File size:" << callbackInfo->FileSize.QuadPart;
+    qDebug(lcCfApiWrapper) << "Desktop client proccess id:" << QCoreApplication::applicationPid();
+    qDebug(lcCfApiWrapper) << "Fetch placeholders requested by proccess id:" << callbackInfo->ProcessInfo->ProcessId;
+    qDebug(lcCfApiWrapper) << "Fetch placeholders requested by application id:" << QString(QString::fromWCharArray(callbackInfo->ProcessInfo->ApplicationId));
+    qDebug(lcCfApiWrapper) << "Fetch placeholders requested for path" << path;
+    if (callbackParameters->FetchPlaceholders.Pattern) {
+        qDebug(lcCfApiWrapper) << "Fetch placeholders requested with pattern:" << QString(QString::fromWCharArray(callbackParameters->FetchPlaceholders.Pattern));
+    }
+
+    const auto sendTransferError = [=] {
+        cfApiSendPlaceholdersTransferInfo(callbackInfo->ConnectionKey,
+                                          callbackInfo->TransferKey,
+                                          STATUS_UNSUCCESSFUL,
+                                          {},
+                                          0,
+                                          0,
+                                          {});
+    };
+
+    const auto sendTransferInfo = [=](const QList<OCC::PlaceholderCreateInfo> &newEntries, const QString &serverPath) {
+        cfApiSendPlaceholdersTransferInfo(callbackInfo->ConnectionKey,
+                                          callbackInfo->TransferKey,
+                                          STATUS_SUCCESS,
+                                          newEntries,
+                                          newEntries.size(),
+                                          newEntries.size(),
+                                          serverPath);
+    };
+
+    auto vfs = reinterpret_cast<OCC::VfsCfApi *>(callbackInfo->CallbackContext);
+    Q_ASSERT(vfs->metaObject()->className() == QByteArrayLiteral("OCC::VfsCfApi"));
+    const auto requestId = QString::number(callbackInfo->TransferKey.QuadPart, 16);
+
+    if (QCoreApplication::applicationPid() == callbackInfo->ProcessInfo->ProcessId) {
+        qCCritical(lcCfApiWrapper) << "implicit hydration triggered by the client itself. Will lead to a deadlock. Cancel" << path << requestId;
+        sendTransferError();
+        return;
+    }
+
+    auto pathString = QFileInfo{path}.canonicalFilePath();
+    auto rootPath = QFileInfo{vfs->params().filesystemPath}.canonicalFilePath();
+
+    if (!pathString.startsWith(rootPath)) {
+        qCCritical(lcCfApiWrapper) << "wrong path" << pathString << rootPath;
+        sendTransferError();
+        return;
+    }
+    const auto remoteSyncRootPath = vfs->params().remotePath; // with leading slash
+    const auto serverPath = QString{remoteSyncRootPath + pathString.mid(rootPath.length() + 1)}.mid(1);
+
+    // to allow navigating to a folder if it's still a virtual directory:
+    qCDebug(lcCfApiWrapper) << "sending empty placeholders for" << path << serverPath << requestId;
+    sendTransferInfo({}, serverPath);
+
+#if 0
+    qCDebug(lcCfApiWrapper) << "fetch placeholder:" << path << serverPath << requestId;
+
+    QEventLoop localEventLoop;
+
+    auto lsPropPromise = QPromise<OCC::PlaceholderCreateInfo>{};
+    auto lsPropFuture = lsPropPromise.future();
+    auto lsPropFutureWatcher = QFutureWatcher<OCC::PlaceholderCreateInfo>{};
+    lsPropFutureWatcher.setFuture(lsPropFuture);
+
+    QList<OCC::PlaceholderCreateInfo> newEntries;
+
+    QObject::connect(&lsPropFutureWatcher, &QFutureWatcher<QStringList>::finished,
+                     &localEventLoop, [&localEventLoop] () {
+                         qCInfo(lcCfApiWrapper()) << "ls prop finished";
+                         localEventLoop.quit();
+                     });
+
+    QObject::connect(&lsPropFutureWatcher, &QFutureWatcher<QStringList>::resultReadyAt,
+                     &localEventLoop, [&newEntries, &lsPropFutureWatcher] (int resultIndex) {
+                         qCInfo(lcCfApiWrapper()) << "ls prop result at index" << resultIndex;
+                         const auto &newResultEntries = lsPropFutureWatcher.resultAt(resultIndex);
+                         newEntries.append(newResultEntries);
+                     });
+
+    QObject::connect(&lsPropFutureWatcher, &QFutureWatcher<QStringList>::started,
+                     &localEventLoop, [] () {
+                         qCInfo(lcCfApiWrapper()) << "ls prop started";
+                     });
+
+    QMetaObject::invokeMethod(vfs->params().account.data(), &OCC::Account::listRemoteFolder, &lsPropPromise, remoteSyncRootPath, serverPath, vfs->params().journal);
+
+    qCInfo(lcCfApiWrapper()) << "ls prop requested" << path << serverPath;
+
+    localEventLoop.exec();
+
+    qCInfo(lcCfApiWrapper()) << "ls prop finished" << path << serverPath << "discovered new entries:" << newEntries.size();
+
+    sendTransferInfo(newEntries, serverPath);
+
+    auto newPlaceholdersResult = 0;
+    const auto invokeFinalizeResult = QMetaObject::invokeMethod(vfs,
+                                                                [&newPlaceholdersResult, vfs, &newEntries, &serverPath] () -> int { return vfs->finalizeNewPlaceholders(newEntries, serverPath); },
+                                                                Qt::BlockingQueuedConnection,
+                                                                qReturnArg(newPlaceholdersResult));
+    if (!invokeFinalizeResult) {
+        qCritical(lcCfApiWrapper) << "Failed to finalize hydration job for" << path << requestId;
+        sendTransferError();
+    }
+    qCInfo(lcCfApiWrapper) << "call for finalizeNewPlaceholders was done";
+
+    if (!newPlaceholdersResult) {
+        sendTransferError();
+    }
+#endif
+    qCInfo(lcCfApiWrapper) << "call for finalizeNewPlaceholders succeeded";
+}
+
 void CALLBACK cfApiNotifyFileCloseCompletion(const CF_CALLBACK_INFO *callbackInfo, const CF_CALLBACK_PARAMETERS * /*callbackParameters*/)
 {
     const auto path = QString(QString::fromWCharArray(callbackInfo->VolumeDosName) + QString::fromWCharArray(callbackInfo->NormalizedPath));
@@ -450,6 +645,7 @@ CF_CALLBACK_REGISTRATION cfApiCallbacks[] = {
     { CF_CALLBACK_TYPE_NOTIFY_FILE_OPEN_COMPLETION, cfApiNotifyFileOpenCompletion },
     { CF_CALLBACK_TYPE_NOTIFY_FILE_CLOSE_COMPLETION, cfApiNotifyFileCloseCompletion },
     { CF_CALLBACK_TYPE_VALIDATE_DATA, cfApiValidateData },
+    { CF_CALLBACK_TYPE_FETCH_PLACEHOLDERS, cfApiFetchPlaceHolders },
     { CF_CALLBACK_TYPE_CANCEL_FETCH_PLACEHOLDERS, cfApiCancelFetchPlaceHolders },
     CF_CALLBACK_REGISTRATION_END
 };
@@ -680,7 +876,7 @@ OCC::Result<void, QString> OCC::CfApiWrapper::registerSyncRoot(const QString &pa
     policies.StructSize = sizeof(CF_SYNC_POLICIES);
     policies.Hydration.Primary = CF_HYDRATION_POLICY_FULL;
     policies.Hydration.Modifier = CF_HYDRATION_POLICY_MODIFIER_NONE;
-    policies.Population.Primary = CF_POPULATION_POLICY_ALWAYS_FULL;
+    policies.Population.Primary = CF_POPULATION_POLICY_ALWAYS_FULL; // CF_POPULATION_POLICY_PARTIAL;
     policies.Population.Modifier = CF_POPULATION_POLICY_MODIFIER_NONE;
     policies.InSync = CF_INSYNC_POLICY_PRESERVE_INSYNC_FOR_SYNC_ENGINE;
     policies.HardLink = CF_HARDLINK_POLICY_NONE;
@@ -828,6 +1024,10 @@ OCC::CfApiWrapper::FileHandle OCC::CfApiWrapper::handleForPath(const QString &pa
 
 OCC::CfApiWrapper::PlaceHolderInfo OCC::CfApiWrapper::findPlaceholderInfo(const QString &path)
 {
+    if (!FileSystem::fileExists(path)) {
+        return {};
+    }
+
     if (auto handle = handleForPath(path)) {
         std::vector<char> buffer(512);
         DWORD actualSize = {};
@@ -906,9 +1106,72 @@ OCC::Result<void, QString> OCC::CfApiWrapper::createPlaceholderInfo(const QStrin
     return {};
 }
 
-OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::updatePlaceholderInfo(const QString &path, time_t modtime, qint64 size, const QByteArray &fileId, const QString &replacesPath)
+OCC::Result<void, QString> OCC::CfApiWrapper::createPlaceholdersInfo(const QString &localBasePath, const QList<PlaceholdersInfo> &itemsInfo)
 {
-    return updatePlaceholderState(path, modtime, size, fileId, replacesPath, CfApiUpdateMetadataType::AllMetadata);
+    auto filteredItemsInfo = QList<PlaceholdersInfo>{};
+    filteredItemsInfo.reserve(itemsInfo.size());
+
+    std::copy_if(itemsInfo.begin(), itemsInfo.end(), std::back_inserter(filteredItemsInfo), [] (const auto &onePlaceholderInfo) -> bool {
+        if (onePlaceholderInfo.modtime <= 0) {
+            qCWarning(lcCfApiWrapper()) << "Skip invalid modtime file: " << onePlaceholderInfo.relativePath << "modtime:" << onePlaceholderInfo.modtime;
+            return false;
+        }
+
+        return true;
+    });
+    const auto stdWStringBasePath = localBasePath.toStdWString();
+    auto cloudEntry = std::make_unique<CF_PLACEHOLDER_CREATE_INFO[]>(filteredItemsInfo.size());
+
+    for(auto itemIndice = 0; itemIndice < filteredItemsInfo.size(); ++itemIndice) {
+        const auto &placeholderInfo = filteredItemsInfo[itemIndice];
+
+        cloudEntry[itemIndice].FileIdentity = placeholderInfo.fileId.data();
+        cloudEntry[itemIndice].FileIdentityLength = static_cast<DWORD>(placeholderInfo.fileId.length());
+
+        cloudEntry[itemIndice].RelativeFileName = placeholderInfo.platformNativeRelativePath.data();
+        cloudEntry[itemIndice].Flags = CF_PLACEHOLDER_CREATE_FLAG_MARK_IN_SYNC;
+        cloudEntry[itemIndice].FsMetadata.FileSize.QuadPart = placeholderInfo.size;
+        cloudEntry[itemIndice].FsMetadata.BasicInfo.FileAttributes = FILE_ATTRIBUTE_NORMAL;
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(placeholderInfo.modtime, &cloudEntry[itemIndice].FsMetadata.BasicInfo.CreationTime);
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(placeholderInfo.modtime, &cloudEntry[itemIndice].FsMetadata.BasicInfo.LastWriteTime);
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(placeholderInfo.modtime, &cloudEntry[itemIndice].FsMetadata.BasicInfo.LastAccessTime);
+        OCC::Utility::UnixTimeToLargeIntegerFiletime(placeholderInfo.modtime, &cloudEntry[itemIndice].FsMetadata.BasicInfo.ChangeTime);
+
+        if (placeholderInfo.fileInfo.isDir()) {
+            cloudEntry[itemIndice].Flags |= CF_PLACEHOLDER_CREATE_FLAG_DISABLE_ON_DEMAND_POPULATION;
+            cloudEntry[itemIndice].FsMetadata.BasicInfo.FileAttributes = FILE_ATTRIBUTE_DIRECTORY;
+            cloudEntry[itemIndice].FsMetadata.FileSize.QuadPart = 0;
+        }
+    }
+
+    auto numberOfCreatedPlaceholders = 0ul;
+    const qint64 result = CfCreatePlaceholders(stdWStringBasePath.data(), cloudEntry.get(), filteredItemsInfo.size(), CF_CREATE_FLAG_NONE, &numberOfCreatedPlaceholders);
+    if (result != S_OK) {
+        qCWarning(lcCfApiWrapper) << "Couldn't create placeholders info" << ":" << QString::fromWCharArray(_com_error(result).ErrorMessage()) << "number of placeholders created:" << numberOfCreatedPlaceholders;
+
+        for(auto itemIndice = 0; itemIndice < filteredItemsInfo.size(); ++itemIndice) {
+            qCDebug(lcCfApiWrapper) << QString::fromStdWString(cloudEntry[itemIndice].RelativeFileName) << QString::fromWCharArray(_com_error(cloudEntry[itemIndice].Result).ErrorMessage());
+        }
+
+        return { "Couldn't create placeholder info" };
+    }
+
+    for(auto itemIndice = 0; itemIndice < filteredItemsInfo.size(); ++itemIndice) {
+        const auto &placeholderInfo = filteredItemsInfo[itemIndice];
+        const auto parentInfo = findPlaceholderInfo(QDir::toNativeSeparators(QFileInfo(localBasePath + QDir::separator() + placeholderInfo.relativePath).absolutePath()));
+        const auto state = parentInfo && parentInfo->PinState == CF_PIN_STATE_UNPINNED ? CF_PIN_STATE_UNPINNED : CF_PIN_STATE_INHERIT;
+
+        if (!setPinState(QDir::toNativeSeparators(QFileInfo(localBasePath + QDir::separator() + placeholderInfo.relativePath).absoluteFilePath()), cfPinStateToPinState(state), NoRecurse)) {
+            return { "Couldn't set the default inherit pin state" };
+        }
+    }
+
+    return {};
+}
+
+OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::updatePlaceholderInfo(const QString &path, const SyncFileItem &item, const QString &replacesPath)
+{
+    return updatePlaceholderState(path, item, replacesPath, CfApiUpdateMetadataType::AllMetadata);
 }
 
 OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::dehydratePlaceholder(const QString &path, time_t modtime, qint64 size, const QByteArray &fileId)
@@ -945,13 +1208,16 @@ OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::de
     return OCC::Vfs::ConvertToPlaceholderResult::Ok;
 }
 
-OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::convertToPlaceholder(const QString &path, time_t modtime, qint64 size, const QByteArray &fileId, const QString &replacesPath)
+OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::convertToPlaceholder(const QString &path, const SyncFileItem &item, const QString &replacesPath)
 {
-    Q_UNUSED(modtime);
-    Q_UNUSED(size);
+    const QByteArray &fileId = item._fileId;
+    const auto fileIdentity = QString::fromUtf8(fileId).toStdWString();
+    const auto fileIdentitySize = (fileIdentity.length() + 1) * sizeof(wchar_t);
+    // const auto createPlaceholderFlags = CF_CONVERT_FLAG_MARK_IN_SYNC | (item.isDirectory() ? (item._type == ItemType::ItemTypeVirtualDirectory ? CF_CONVERT_FLAG_ENABLE_ON_DEMAND_POPULATION : CF_CONVERT_FLAG_ALWAYS_FULL) : CF_CONVERT_FLAG_MARK_IN_SYNC);
+    const auto directoryPlaceholderFlags = item.isDirectory() ? CF_CONVERT_FLAG_ALWAYS_FULL : CF_CONVERT_FLAG_NONE;
+    const auto createPlaceholderFlags = CF_CONVERT_FLAG_MARK_IN_SYNC | directoryPlaceholderFlags;
 
-    const qint64 result =
-        CfConvertToPlaceholder(handleForPath(path).get(), fileId.data(), static_cast<DWORD>(fileId.size()), CF_CONVERT_FLAG_MARK_IN_SYNC, nullptr, nullptr);
+    const auto result = CfConvertToPlaceholder(handleForPath(path).get(), fileIdentity.data(), sizeToDWORD(fileIdentitySize), createPlaceholderFlags, nullptr, nullptr);
     Q_ASSERT(result == S_OK);
     if (result != S_OK) {
         const auto errorMessage = createErrorMessageForPlaceholderUpdateAndCreate(path, "Couldn't convert to placeholder");
@@ -972,9 +1238,9 @@ OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::co
     return stateResult;
 }
 
-OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::updatePlaceholderMarkInSync(const QString &path, const QByteArray &fileId, const QString &replacesPath)
+OCC::Result<OCC::Vfs::ConvertToPlaceholderResult, QString> OCC::CfApiWrapper::updatePlaceholderMarkInSync(const QString &path, const SyncFileItem &item, const QString &replacesPath)
 {
-    return updatePlaceholderState(path, {}, {}, fileId, replacesPath, CfApiUpdateMetadataType::OnlyBasicMetadata);
+    return updatePlaceholderState(path, item, replacesPath, CfApiUpdateMetadataType::OnlyBasicMetadata);
 }
 
 bool OCC::CfApiWrapper::isPlaceHolderInSync(const QString &filePath)
