@@ -1,8 +1,11 @@
 /*
- *    This software is in the public domain, furnished "as is", without technical
- *    support, and with no warranty, express or implied, as to its usefulness for
- *    any purpose.
+ * SPDX-FileCopyrightText: 2020 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2016 ownCloud GmbH
+ * SPDX-License-Identifier: CC0-1.0
  *
+ * This software is in the public domain, furnished "as is", without technical
+ * support, and with no warranty, express or implied, as to its usefulness for
+ * any purpose.
  */
 
 #include <QtTest>
@@ -118,9 +121,9 @@ private slots:
     {
         setup();
         excludedFiles->addManualExclude("*", "/tmp/check_csync1/");
-        QCOMPARE(check_file_full("/tmp/check_csync1/foo"), CSYNC_FILE_EXCLUDE_LIST);
+        QCOMPARE(check_file_full("/tmp/check_csync1/foo"), CSYNC_NOT_EXCLUDED);
         QCOMPARE(check_file_full("/tmp/check_csync2/foo"), CSYNC_NOT_EXCLUDED);
-        QVERIFY(excludedFiles->_allExcludes[QStringLiteral("/tmp/check_csync1/")].contains("*"));
+        QVERIFY(!excludedFiles->_allExcludes[QStringLiteral("/tmp/check_csync1/")].contains("*"));
 
         excludedFiles->addManualExclude("foo");
         QVERIFY(excludedFiles->_fullRegexFile[QStringLiteral("/")].pattern().contains("foo"));
@@ -715,7 +718,7 @@ private slots:
     void testAddExcludeFilePath_addSameFilePath_listSizeDoesNotIncrease()
     {
         excludedFiles.reset(new ExcludedFiles());
-        const auto filePath = QString("exclude/.sync-exclude.lst");
+        const auto filePath = QStringLiteral("exclude/.sync-exclude.lst");
         
         excludedFiles->addExcludeFilePath(filePath);
         excludedFiles->addExcludeFilePath(filePath);        
@@ -727,8 +730,8 @@ private slots:
     {
         excludedFiles.reset(new ExcludedFiles());
         
-        const auto filePath1 = QString("exclude1/.sync-exclude.lst");
-        const auto filePath2 = QString("exclude2/.sync-exclude.lst");
+        const auto filePath1 = QStringLiteral("exclude1/.sync-exclude.lst");
+        const auto filePath2 = QStringLiteral("exclude2/.sync-exclude.lst");
     
         excludedFiles->addExcludeFilePath(filePath1);
         excludedFiles->addExcludeFilePath(filePath2);
