@@ -1,8 +1,11 @@
 /*
- *    This software is in the public domain, furnished "as is", without technical
- *    support, and with no warranty, express or implied, as to its usefulness for
- *    any purpose.
+ * SPDX-FileCopyrightText: 2021 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2018 ownCloud GmbH
+ * SPDX-License-Identifier: CC0-1.0
  *
+ * This software is in the public domain, furnished "as is", without technical
+ * support, and with no warranty, express or implied, as to its usefulness for
+ * any purpose.
  */
 
 #include <QtTest>
@@ -11,6 +14,7 @@
 #include <localdiscoverytracker.h>
 
 using namespace OCC;
+using namespace Qt::StringLiterals;
 
 class TestLocalDiscovery : public QObject
 {
@@ -333,16 +337,55 @@ private slots:
         QVERIFY(!fakeFolder.currentRemoteState().find("C/filename.ext"));
     }
 
-    void testCreateFileWithTrailingSpaces_localAndRemoteTrimmedDoNotExist_renameAndUploadFile()
+    void testRedownloadDeletedLivePhotoMov()
     {
         FakeFolder fakeFolder{FileInfo{}};
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
+        const auto livePhotoImg = QStringLiteral("IMG_0001.heic");
+        const auto livePhotoMov = QStringLiteral("IMG_0001.mov");
+        fakeFolder.localModifier().insert(livePhotoImg);
+        fakeFolder.localModifier().insert(livePhotoMov);
+
+        ItemCompletedSpy completeSpy(fakeFolder);
+        QVERIFY(fakeFolder.syncOnce());
+
+        QCOMPARE(completeSpy.findItem(livePhotoImg)->_status, SyncFileItem::Status::Success);
+        QCOMPARE(completeSpy.findItem(livePhotoMov)->_status, SyncFileItem::Status::Success);
+
+        fakeFolder.remoteModifier().setIsLivePhoto(livePhotoImg, true);
+        fakeFolder.remoteModifier().setIsLivePhoto(livePhotoMov, true);
+        QVERIFY(fakeFolder.syncOnce());
+
+        SyncJournalFileRecord imgRecord;
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(livePhotoImg, &imgRecord));
+        QVERIFY(imgRecord._isLivePhoto);
+
+        SyncJournalFileRecord movRecord;
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(livePhotoMov, &movRecord));
+        QVERIFY(movRecord._isLivePhoto);
+
+        completeSpy.clear();
+        fakeFolder.localModifier().remove(livePhotoMov);
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(completeSpy.findItem(livePhotoMov)->_status, SyncFileItem::Status::Success);
+        QCOMPARE(completeSpy.findItem(livePhotoMov)->_instruction, CSYNC_INSTRUCTION_SYNC);
+        QCOMPARE(completeSpy.findItem(livePhotoMov)->_direction, SyncFileItem::Direction::Down);
+    }
+
+    void testCreateFileWithTrailingSpaces_localAndRemoteTrimmedDoNotExist_renameAndUploadFile()
+    {
+        FakeFolder fakeFolder{FileInfo{}};
+        fakeFolder.enableEnforceWindowsFileNameCompatibility();
+
+        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
+
         const QString fileWithSpaces1(" foo");
         const QString fileWithSpaces2(" bar ");
         const QString fileWithSpaces3("bla ");
         const QString fileWithSpaces4("A/ foo");
         const QString fileWithSpaces5("A/ bar ");
         const QString fileWithSpaces6("A/bla ");
+        const auto extraFileNameWithSpaces = QStringLiteral(" with spaces ");
 
         fakeFolder.localModifier().insert(fileWithSpaces1);
         fakeFolder.localModifier().insert(fileWithSpaces2);
@@ -351,30 +394,20 @@ private slots:
         fakeFolder.localModifier().insert(fileWithSpaces4);
         fakeFolder.localModifier().insert(fileWithSpaces5);
         fakeFolder.localModifier().insert(fileWithSpaces6);
-        fakeFolder.localModifier().mkdir(QStringLiteral(" with spaces "));
+        fakeFolder.localModifier().mkdir(extraFileNameWithSpaces);
 
         ItemCompletedSpy completeSpy(fakeFolder);
         completeSpy.clear();
 
         QVERIFY(fakeFolder.syncOnce());
 
-#if defined Q_OS_WINDOWS
-        QCOMPARE(completeSpy.findItem(fileWithSpaces1)->_status, SyncFileItem::Status::FileNameInvalid);
+        QCOMPARE(completeSpy.findItem(fileWithSpaces1)->_status, SyncFileItem::Status::Success);
         QCOMPARE(completeSpy.findItem(fileWithSpaces2)->_status, SyncFileItem::Status::FileNameInvalid);
         QCOMPARE(completeSpy.findItem(fileWithSpaces3)->_status, SyncFileItem::Status::FileNameInvalid);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::FileNameInvalid);
+        QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::Success);
         QCOMPARE(completeSpy.findItem(fileWithSpaces5)->_status, SyncFileItem::Status::FileNameInvalid);
         QCOMPARE(completeSpy.findItem(fileWithSpaces6)->_status, SyncFileItem::Status::FileNameInvalid);
-        QCOMPARE(completeSpy.findItem(QStringLiteral(" with spaces "))->_status, SyncFileItem::Status::FileNameInvalid);
-#else
-        QCOMPARE(completeSpy.findItem(fileWithSpaces1)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces2)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces3)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces5)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces6)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(QStringLiteral(" with spaces "))->_status, SyncFileItem::Status::Success);
-#endif
+        QCOMPARE(completeSpy.findItem(extraFileNameWithSpaces)->_status, SyncFileItem::Status::FileNameInvalid);
 
         fakeFolder.syncEngine().addAcceptedInvalidFileName(fakeFolder.localPath() + fileWithSpaces1);
         fakeFolder.syncEngine().addAcceptedInvalidFileName(fakeFolder.localPath() + fileWithSpaces2);
@@ -382,21 +415,21 @@ private slots:
         fakeFolder.syncEngine().addAcceptedInvalidFileName(fakeFolder.localPath() + fileWithSpaces4);
         fakeFolder.syncEngine().addAcceptedInvalidFileName(fakeFolder.localPath() + fileWithSpaces5);
         fakeFolder.syncEngine().addAcceptedInvalidFileName(fakeFolder.localPath() + fileWithSpaces6);
-        fakeFolder.syncEngine().addAcceptedInvalidFileName(fakeFolder.localPath() + QStringLiteral(" with spaces "));
+        fakeFolder.syncEngine().addAcceptedInvalidFileName(fakeFolder.localPath() + extraFileNameWithSpaces);
 
         completeSpy.clear();
 
         fakeFolder.syncEngine().setLocalDiscoveryOptions(LocalDiscoveryStyle::DatabaseAndFilesystem, {QStringLiteral("foo"), QStringLiteral("bar"), QStringLiteral("bla"), QStringLiteral("A/foo"), QStringLiteral("A/bar"), QStringLiteral("A/bla")});
         QVERIFY(fakeFolder.syncOnce());
 
-#if defined Q_OS_WINDOWS
-        QCOMPARE(completeSpy.findItem(fileWithSpaces1)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces2)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces3)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces5)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces6)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(QStringLiteral(" with spaces "))->_status, SyncFileItem::Status::NormalError);
+#if !defined Q_OS_WINDOWS
+        QCOMPARE(completeSpy.findItem(fileWithSpaces1)->_status, SyncFileItem::Status::NoStatus);
+        QCOMPARE(completeSpy.findItem(QStringLiteral(" bar"))->_status, SyncFileItem::Status::Success);
+        QCOMPARE(completeSpy.findItem(QStringLiteral("bla"))->_status, SyncFileItem::Status::Success);
+        QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::NoStatus);
+        QCOMPARE(completeSpy.findItem(QStringLiteral("A/ bar"))->_status, SyncFileItem::Status::Success);
+        QCOMPARE(completeSpy.findItem(QStringLiteral("A/bla"))->_status, SyncFileItem::Status::Success);
+        QCOMPARE(completeSpy.findItem(QStringLiteral(" with spaces"))->_status, SyncFileItem::Status::Success);
 #endif
     }
 
@@ -419,20 +452,21 @@ private slots:
 
         QVERIFY(fakeFolder.syncOnce());
 
-        if (Utility::isWindows()) {
-            QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::FileNameInvalid);
+#if defined Q_OS_WINDOWS
+            QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::Success);
             QCOMPARE(completeSpy.findItem(fileWithSpaces5)->_status, SyncFileItem::Status::FileNameInvalid);
             QCOMPARE(completeSpy.findItem(fileWithSpaces6)->_status, SyncFileItem::Status::FileNameInvalid);
-        } else {
+#else
             QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::Success);
             QCOMPARE(completeSpy.findItem(fileWithSpaces5)->_status, SyncFileItem::Status::Success);
             QCOMPARE(completeSpy.findItem(fileWithSpaces6)->_status, SyncFileItem::Status::Success);
-        }
+#endif
     }
 
     void testCreateLocalPathsWithLeadingAndTrailingSpaces_syncOnSupportingOs()
     {
         FakeFolder fakeFolder{FileInfo()};
+        fakeFolder.enableEnforceWindowsFileNameCompatibility();
         fakeFolder.localModifier().mkdir("A");
         QVERIFY(fakeFolder.syncOnce());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
@@ -454,18 +488,16 @@ private slots:
 
         QVERIFY(fakeFolder.syncOnce());
 
-#if !defined Q_OS_WINDOWS
         QCOMPARE(completeSpy.findItem(fileWithSpaces1)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces2)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(fileWithSpaces3)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(folderWithSpaces1)->_status, SyncFileItem::Status::Success);
-        QCOMPARE(completeSpy.findItem(folderWithSpaces2)->_status, SyncFileItem::Status::Success);
+        QCOMPARE(completeSpy.findItem(fileWithSpaces2)->_status, SyncFileItem::Status::FileNameInvalid);
+        QCOMPARE(completeSpy.findItem(fileWithSpaces3)->_status, SyncFileItem::Status::FileNameInvalid);
+        QCOMPARE(completeSpy.findItem(folderWithSpaces1)->_status, SyncFileItem::Status::FileNameInvalid);
+        QCOMPARE(completeSpy.findItem(folderWithSpaces2)->_status, SyncFileItem::Status::FileNameInvalid);
         QVERIFY(fakeFolder.remoteModifier().find(fileWithSpaces1));
-        QVERIFY(fakeFolder.remoteModifier().find(fileWithSpaces2));
-        QVERIFY(fakeFolder.remoteModifier().find(fileWithSpaces3));
-        QVERIFY(fakeFolder.remoteModifier().find(folderWithSpaces1));
-        QVERIFY(fakeFolder.remoteModifier().find(folderWithSpaces2));
-#endif
+        QVERIFY(!fakeFolder.remoteModifier().find(fileWithSpaces2));
+        QVERIFY(!fakeFolder.remoteModifier().find(fileWithSpaces3));
+        QVERIFY(!fakeFolder.remoteModifier().find(folderWithSpaces1));
+        QVERIFY(!fakeFolder.remoteModifier().find(folderWithSpaces2));
     }
 
     void testCreateFileWithTrailingSpaces_remoteGetRenamedManually()
@@ -489,17 +521,17 @@ private slots:
         ItemCompletedSpy completeSpy(fakeFolder);
         completeSpy.clear();
 
-        QVERIFY(fakeFolder.syncOnce());        
+        QVERIFY(fakeFolder.syncOnce());
 
-        if (Utility::isWindows()) {
-            QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::FileNameInvalid);
+#if defined Q_OS_WINDOWS
+            QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::Success);
             QCOMPARE(completeSpy.findItem(fileWithSpaces5)->_status, SyncFileItem::Status::FileNameInvalid);
             QCOMPARE(completeSpy.findItem(fileWithSpaces6)->_status, SyncFileItem::Status::FileNameInvalid);
-        } else {
+#else
             QCOMPARE(completeSpy.findItem(fileWithSpaces4)->_status, SyncFileItem::Status::Success);
             QCOMPARE(completeSpy.findItem(fileWithSpaces5)->_status, SyncFileItem::Status::Success);
             QCOMPARE(completeSpy.findItem(fileWithSpaces6)->_status, SyncFileItem::Status::Success);
-        }
+#endif
 
         fakeFolder.remoteModifier().rename(fileWithSpaces4, fileWithoutSpaces4);
         fakeFolder.remoteModifier().rename(fileWithSpaces5, fileWithoutSpaces5);
@@ -517,8 +549,9 @@ private slots:
     void testCreateFileWithTrailingSpaces_localTrimmedAlsoCreated_dontRenameAutomaticallyAndDontUploadFile()
     {
         FakeFolder fakeFolder{FileInfo{}};
+        fakeFolder.enableEnforceWindowsFileNameCompatibility();
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
-        const QString fileWithSpaces(" foo");
+        const QString fileWithSpaces("foo ");
         const QString fileTrimmed("foo");
 
         fakeFolder.localModifier().insert(fileTrimmed);
@@ -528,13 +561,8 @@ private slots:
 
         QVERIFY(fakeFolder.currentRemoteState().find(fileTrimmed));
 
-#if defined Q_OS_WINDOWS
         // no file with spaces on Windows
         QVERIFY(!fakeFolder.currentRemoteState().find(fileWithSpaces));
-#else
-        //Linux/Mac OS allows spaces
-        QVERIFY(fakeFolder.currentRemoteState().find(fileWithSpaces));
-#endif
 
         QVERIFY(fakeFolder.currentLocalState().find(fileWithSpaces));
         QVERIFY(fakeFolder.currentLocalState().find(fileTrimmed));
@@ -581,9 +609,11 @@ private slots:
 
         QVERIFY(fakeFolder.syncOnce());
 
+#if !defined Q_OS_WINDOWS
         auto expectedState = fakeFolder.currentLocalState();
         qDebug() << expectedState;
         QCOMPARE(fakeFolder.currentRemoteState(), expectedState);
+#endif
     }
 
     void testBlockInvalidMtimeSyncRemote()
@@ -837,6 +867,177 @@ private slots:
         QVERIFY(fakeFolder.syncJournal().getFileRecord(QStringLiteral("bar"), &fileRecordAfter));
         QVERIFY(!fileRecordAfter._lockstate._locked);
     }
+
+    void testDiscoveryUsesCorrectQuotaSource()
+    {
+        //setup sync folder
+        FakeFolder fakeFolder{FileInfo{}};
+
+        // create folder
+        const QString folderA("A");
+        fakeFolder.localModifier().mkdir(folderA);
+        fakeFolder.remoteModifier().mkdir(folderA);
+        fakeFolder.remoteModifier().setFolderQuota(folderA, {0, 500});
+
+        // sync folderA
+        ItemCompletedSpy syncSpy(fakeFolder);
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(folderA)->_status, SyncFileItem::Status::NoStatus);
+
+        // check db quota for folderA - bytesAvailable is 500
+        SyncJournalFileRecord recordFolderA;
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(folderA, &recordFolderA));
+        QCOMPARE(recordFolderA._folderQuota.bytesAvailable, 500);
+
+        // add fileNameA to folderA - size < quota in db
+        const QString fileNameA("A/A.data");
+        fakeFolder.localModifier().insert(fileNameA, 200);
+
+        // set different quota for folderA - remote change does not change etag yet
+        fakeFolder.remoteModifier().setFolderQuota(folderA, {0, 0});
+
+        // sync filenameA - size == quota => success
+        syncSpy.clear();
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(fileNameA)->_status, SyncFileItem::Status::Success);
+
+        // add smallFile to folderA - size < quota in db
+        const QString smallFile("A/smallFile.data");
+        fakeFolder.localModifier().insert(smallFile, 100);
+
+        // sync smallFile - size < quota in db => success => update quota in db
+        syncSpy.clear();
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(smallFile)->_status, SyncFileItem::Status::Success);
+
+        // create remoteFileA - size > bytes available
+        const QString remoteFileA("A/remoteA.data");
+        fakeFolder.remoteModifier().insert(remoteFileA, 200);
+
+        // sync remoteFile - it is a download
+        syncSpy.clear();
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(remoteFileA)->_status, SyncFileItem::Status::Success);
+
+        // check db quota for folderA - bytesAvailable have changed to 0 due to new PROPFIND
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(folderA, &recordFolderA));
+        QCOMPARE(recordFolderA._folderQuota.bytesAvailable, 0);
+
+        // create local fileNameB - size < quota in db
+        const QString fileNameB("A/B.data");
+        fakeFolder.localModifier().insert(fileNameB, 0);
+
+        // set different quota for folderA - remote change does not change etag yet
+        fakeFolder.remoteModifier().setFolderQuota(folderA, {500, 600});
+
+        // sync fileNameB - size < quota in db => success
+        syncSpy.clear();
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(fileNameB)->_status, SyncFileItem::Status::Success);
+
+        // create remoteFileB - it is a download
+        const QString remoteFileB("A/remoteB.data");
+        fakeFolder.remoteModifier().insert(remoteFileA, 100);
+
+        // create local fileNameC - size < quota in db
+        const QString fileNameC("A/C.data");
+        fakeFolder.localModifier().insert(fileNameC, 0);
+
+        // sync filenameC - size < quota in db => success
+        syncSpy.clear();
+        QVERIFY(fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(fileNameC)->_status, SyncFileItem::Status::Success);
+
+        // check db quota for folderA - bytesAvailable have changed to 600 due to new PROPFIND
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(folderA, &recordFolderA));
+        QCOMPARE(recordFolderA._folderQuota.bytesAvailable, 600);
+
+        QCOMPARE(syncSpy.findItem(remoteFileB)->_status, SyncFileItem::Status::NoStatus);
+
+        // create local fileNameD - size > quota in db
+        const QString fileNameD("A/D.data");
+        fakeFolder.localModifier().insert(fileNameD, 700);
+
+        // sync fileNameD - size > quota in db => error
+        syncSpy.clear();
+        QVERIFY(!fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(fileNameD)->_status, SyncFileItem::Status::NormalError);
+
+        // create local fileNameE - size < quota in db
+        const QString fileNameE("A/E.data");
+        fakeFolder.localModifier().insert(fileNameE, 400);
+
+        // sync fileNameE - size < quota in db => success
+        syncSpy.clear();
+        QVERIFY(!fakeFolder.syncOnce());
+        QCOMPARE(syncSpy.findItem(fileNameE)->_status, SyncFileItem::Status::Success);
+    }
+
+    void testMissingInodeAndFixThem()
+    {
+        FakeFolder fakeFolder{FileInfo{}};
+
+        fakeFolder.remoteModifier().mkdir(u"A"_s);
+        fakeFolder.remoteModifier().mkdir(u"A/B"_s);
+        fakeFolder.remoteModifier().insert(u"textFile.txt"_s);
+        fakeFolder.remoteModifier().insert(u"document.md"_s);
+        fakeFolder.remoteModifier().insert(u"A/B/sub-folder.md"_s);
+
+        QVERIFY(fakeFolder.syncOnce());
+
+        auto documentFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"document.md"_s, &documentFileRecord));
+        QCOMPARE_GT(documentFileRecord._inode, 0);
+        documentFileRecord._inode = 0;
+        QVERIFY(fakeFolder.syncJournal().setFileRecord(documentFileRecord).isValid());
+
+        auto textFileFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"textFile.txt"_s, &textFileFileRecord));
+        QCOMPARE_GT(textFileFileRecord._inode, 0);
+        textFileFileRecord._inode = 0;
+        QVERIFY(fakeFolder.syncJournal().setFileRecord(textFileFileRecord).isValid());
+
+        auto aFolderFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"A"_s, &aFolderFileRecord));
+        QCOMPARE_GT(aFolderFileRecord._inode, 0);
+        aFolderFileRecord._inode = 0;
+        QVERIFY(fakeFolder.syncJournal().setFileRecord(aFolderFileRecord).isValid());
+
+        auto bFolderFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"A/B"_s, &bFolderFileRecord));
+        QCOMPARE_GT(bFolderFileRecord._inode, 0);
+        bFolderFileRecord._inode = 0;
+        QVERIFY(fakeFolder.syncJournal().setFileRecord(bFolderFileRecord).isValid());
+
+        auto subFolderFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"A/B/sub-folder.md"_s, &subFolderFileRecord));
+        QCOMPARE_GT(subFolderFileRecord._inode, 0);
+        subFolderFileRecord._inode = 0;
+        QVERIFY(fakeFolder.syncJournal().setFileRecord(subFolderFileRecord).isValid());
+
+        fakeFolder.syncEngine().setLocalDiscoveryOptions(LocalDiscoveryStyle::FilesystemOnly);
+        QVERIFY(fakeFolder.syncOnce());
+
+        documentFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"document.md"_s, &documentFileRecord));
+        QCOMPARE_GT(documentFileRecord._inode, 0);
+
+        textFileFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"textFile.txt"_s, &textFileFileRecord));
+        QCOMPARE_GT(textFileFileRecord._inode, 0);
+
+        aFolderFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"A"_s, &aFolderFileRecord));
+        QCOMPARE_GT(aFolderFileRecord._inode, 0);
+
+        bFolderFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"A/B"_s, &bFolderFileRecord));
+        QCOMPARE_GT(bFolderFileRecord._inode, 0);
+
+        subFolderFileRecord = SyncJournalFileRecord{};
+        QVERIFY(fakeFolder.syncJournal().getFileRecord(u"A/B/sub-folder.md"_s, &subFolderFileRecord));
+        QCOMPARE_GT(subFolderFileRecord._inode, 0);
+    }
 };
 
 QTEST_GUILESS_MAIN(TestLocalDiscovery)
